local _, EpsiIskarAssist = ...
LibStub("AceAddon-3.0"):NewAddon(EpsiIskarAssist, "EpsiIskarAssist", "AceConsole-3.0", "AceEvent-3.0")

EpsiIskarAssist:SetDefaultModuleLibraries("AceConsole-3.0", "AceEvent-3.0")
-- Local variable to speedup things
local UnitGUID = UnitGUID

local ADDON_NAME = "Epsilon Iskar Assist"

-- *****************
-- *** Addon Version
-- *** 1.0 (Alpha)
-- *****************
EpsiIskarAssist.MajorVersion = 1
EpsiIskarAssist.MinorVersion = 0
-- Stage Number
-- 0 : Alpha
-- 1 : Beta
-- 2 : Release candidate
-- 3 : Final release
EpsiIskarAssist.StageVersion = 0
EpsiIskarAssist.RevisionVersion = 0

-- ********************
-- *** Common Data ****
-- ********************
EpsiIskarAssist.IskarID = 90316
EpsiIskarAssist.IskarEncounterID = 1788

EpsiIskarAssist.AuraEyeOfAnzuSpellID = 179202
EpsiIskarAssist.AuraPhantasmalWindsSpellID = 181957
EpsiIskarAssist.AuraPhantasmalWoundsSpellID = 182325
EpsiIskarAssist.AuraPhantasmalFelBombSpellID = 179219
EpsiIskarAssist.AuraFelBombSpellID = 181753
EpsiIskarAssist.AuraPhantasmalCorruptionSpellID = 181824
EpsiIskarAssist.AuraDarkBindingsSpellID = 185510
EpsiIskarAssist.AuraRadianceOfAnzuSpellID = 185239

local Healers = {}
local Tanks = {}
local Dps = {}

local HealerCount = 0
local TankCount = 0
local DpsCount = 0

local Members = {} -- Members[GUID] = raidID

-- ****************************
-- *** Local Misc Functions ***
-- ****************************
local function GetNumMember()
		local num = GetNumGroupMembers()

		difficulty = GetRaidDifficultyID()

		-- Raid Normal
		if difficulty == 14 then
			maxNumber = 30
		-- Raid Heroic
		elseif difficulty == 15 then
			maxNumber = 30
		-- Raid Mythic
		elseif difficulty == 16 then
			maxNumber = 20
		else
			maxNumber = 40
		end

		if num > maxNumber then
			return maxNumber
		else
			return num
		end
end

function EpsiIskarAssist:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EpsiIskarAssistDB")

	self:SetupOptions()

end

function EpsiIskarAssist:OnEnable()
  self:RegisterEvent("PARTY_MEMBERS_CHANGED", "UpdateRaidInfo")
  self:RegisterEvent("GROUP_ROSTER_UPDATE", "UpdateRaidInfo")
  self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateRaidInfo")
  self:RegisterEvent("UNIT_NAME_UPDATE", "UpdateRaidInfo")
end

function EpsiIskarAssist:OnDisable()
  self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
  self:UnregisterEvent("GROUP_ROSTER_UPDATE")
  self:UnregisterEvent("PLAYER_ENTERING_WORLD")
  self:UnregisterEvent("UNIT_NAME_UPDATE")
end

function EpsiIskarAssist:UpdateRaidInfo()
		self:WipeUnits()

    for i = 1, GetNumMember() do
      local unit = string.format("raid%i", i)
      local guid = UnitGUID(unit)
      local name = UnitName(unit)

      if guid and name ~= UNKNOWNOBJECT then
        local role = UnitGroupRolesAssigned(unit)
				--self:Print("Add Unit", unit, guid, role)
				self:AddUnit(unit, guid, role)
      end

    end

		self:SendMessage("EIA_RAID_INFO_UPDATED")
end

function EpsiIskarAssist:WipeUnits()
		wipe(Members)

		wipe(Healers)
		HealerCount = 0

		wipe(Tanks)
		TankCount = 0

		wipe(Dps)
		DpsCount = 0
end


function EpsiIskarAssist:AddUnit(unit, guid, role)
  if role == "HEALER" then
    self:AddHealer(unit, guid)
  elseif role == "TANK" then
    self:AddTank(unit, guid)
  elseif role == "DAMAGER" then
    self:AddDps(unit, guid)
  end
end

function EpsiIskarAssist:AddHealer(unit, guid)
  Members[guid] = unit
  tinsert(Healers, guid)
	HealerCount = HealerCount + 1
end

function EpsiIskarAssist:AddTank(unit, guid)
  Members[guid] = unit
  tinsert(Tanks, guid)
	TankCount = TankCount + 1
end

function EpsiIskarAssist:AddDps(unit, guid)
  Members[guid] = unit
  tinsert(Dps, guid)
	DpsCount = DpsCount + 1
end

function EpsiIskarAssist:GetHealers()
	return Healers, HealerCount
end

function EpsiIskarAssist:GetTanks()
	return Tanks, TankCount
end

function EpsiIskarAssist:GetDps()
	return Dps, DpsCount
end


function EpsiIskarAssist:GetUnit(guid)
  return Members[guid]
end


function EpsiIskarAssist:GetVersionString()
	local stage = ""
	if StageVersion == 0 then
		stage = "Alpha"
	elseif StageVersion == 1 then
		stage = "Beta"
	elseif StageVersion == 2 then
		stage = "Release Candidate"
	elseif StageVersion == 3 then
		stage = "Release"
	end

	local version = string.format("%i.%i.%i.%i (%s)", MajorVersion, MinorVersion, StageVersion, RevisionVersion, stage)

	return version
end

function EpsiIskarAssist:GetNpcID(guid)
	if not guid then return -1 end

	local _, _, _, _, _, npcID, _ = strsplit("-", guid)

	return npcID
end


function EpsiIskarAssist:GetNpcIDFromUnit(unit)
	if not unit then return -1 end

	local guid = UnitGUID(unit)

	return self:GetNpcID(guid)
end

function EpsiIskarAssist:GetIconText(icon, width, height)
	if not icon then return "" end

	if not width or not height then
		height = 24
		width = 24

	end

	return string.format("|T%s:%i:%i|t", icon, width, height)
end

function EpsiIskarAssist:GetRoleIconText(role, width, height)
  if not height or not width then
    height = 32
    width = 32
  end

  if not role or role == "NONE" then
        return ""
    elseif role == "HEALER" then
        return string.format("|T%s:%i:%i:0:0:256:256:69:130:2:63|t", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", width, height)
    elseif role == "DAMAGER" then
        return string.format("|T%s:%i:%i:0:0:256:256:69:130:69:130|t", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", width, height)
    elseif role == "TANK" then
        return string.format("|T%s:%i:%i:0:0:256:256:3:64:69:130|t", "Interface/LFGFRAME/UI-LFG-ICON-ROLES", width, height)
    end
end

function EpsiIskarAssist:GenerateOptions()

	local modulesOptions = {}
	for name, module in self:IterateModules() do
			if module.GetOptions then
				modulesOptions[name] = module:GetOptions()
			end
	end

	EpsiIskarAssist.options = {
		type = "group",
		name = "EpsiIskarAssist",
		childGroups = "tree",
		args = {
			eia = {
				type = "group",
				name = "General",
				order = 1,
				args = {},
			}
		},
		plugins = modulesOptions,

	}
end

function EpsiIskarAssist:GetOptions()
	if not EpsiIskarAssist.options then
		self:GenerateOptions()
	end
	return EpsiIskarAssist.options
end

function EpsiIskarAssist:SetupOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("EpsiIskarAssist", self:GetOptions())
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("EpsiIskarAssist", "EpsiIskarAssist")
end
